@@ -142,10 +142,10 @@ gst_jpegenc_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_jpegenc_sink_pad_template));
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_jpegenc_src_pad_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_jpegenc_sink_pad_template);
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_jpegenc_src_pad_template);
   gst_element_class_set_details_simple (element_class, "JPEG image encoder",
       "Codec/Encoder/Image",
       "Encode images in JPEG format", "Wim Taymans <wim.taymans@tvd.be>");
@@ -173,19 +173,22 @@ gst_jpegenc_class_init (GstJpegEnc * klass)
 
   g_object_class_install_property (gobject_class, PROP_QUALITY,
       g_param_spec_int ("quality", "Quality", "Quality of encoding",
-          0, 100, JPEG_DEFAULT_QUALITY, G_PARAM_READWRITE));
+          0, 100, JPEG_DEFAULT_QUALITY,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 #ifdef ENABLE_SMOOTHING
   /* disabled, since it doesn't seem to work */
   g_object_class_install_property (gobject_class, PROP_SMOOTHING,
       g_param_spec_int ("smoothing", "Smoothing", "Smoothing factor",
-          0, 100, JPEG_DEFAULT_SMOOTHING, G_PARAM_READWRITE));
+          0, 100, JPEG_DEFAULT_SMOOTHING,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 #endif
 
   g_object_class_install_property (gobject_class, PROP_IDCT_METHOD,
       g_param_spec_enum ("idct-method", "IDCT Method",
           "The IDCT algorithm to use", GST_TYPE_IDCT_METHOD,
-          JPEG_DEFAULT_IDCT_METHOD, G_PARAM_READWRITE));
+          JPEG_DEFAULT_IDCT_METHOD,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstelement_class->change_state = gst_jpegenc_change_state;
 
@@ -243,8 +246,7 @@ gst_jpegenc_term_destination (j_compress_ptr cinfo)
 
   /* Trim the buffer size and push it. */
   GST_BUFFER_SIZE (jpegenc->output_buffer) =
-      GST_ROUND_UP_4 (GST_BUFFER_SIZE (jpegenc->output_buffer) -
-      jpegenc->jdest.free_in_buffer);
+      GST_BUFFER_SIZE (jpegenc->output_buffer) - jpegenc->jdest.free_in_buffer;
 
   g_signal_emit (G_OBJECT (jpegenc), gst_jpegenc_signals[FRAME_ENCODED], 0);
 
@@ -342,7 +344,7 @@ gst_jpegenc_getcaps (GstPad * pad)
   /* we want to proxy properties like width, height and framerate from the
      other end of the element */
 
-  othercaps = gst_pad_get_allowed_caps (jpegenc->srcpad);
+  othercaps = gst_pad_peer_get_caps_reffed (jpegenc->srcpad);
   if (othercaps == NULL ||
       gst_caps_is_empty (othercaps) || gst_caps_is_any (othercaps)) {
     caps = gst_caps_copy (gst_pad_get_pad_template_caps (pad));
@@ -517,11 +519,12 @@ gst_jpegenc_resync (GstJpegEnc * jpegenc)
         jpegenc->h_samp[i], jpegenc->v_samp[i]);
     jpegenc->cinfo.comp_info[i].h_samp_factor = jpegenc->h_samp[i];
     jpegenc->cinfo.comp_info[i].v_samp_factor = jpegenc->v_samp[i];
-    jpegenc->line[i] = g_realloc (jpegenc->line[i],
-        jpegenc->v_max_samp * DCTSIZE * sizeof (char *));
+    g_free (jpegenc->line[i]);
+    jpegenc->line[i] = g_new (guchar *, jpegenc->v_max_samp * DCTSIZE);
     if (!jpegenc->planar) {
       for (j = 0; j < jpegenc->v_max_samp * DCTSIZE; j++) {
-        jpegenc->row[i][j] = g_realloc (jpegenc->row[i][j], width);
+        g_free (jpegenc->row[i][j]);
+        jpegenc->row[i][j] = g_malloc (width);
         jpegenc->line[i][j] = jpegenc->row[i][j];
       }
     }
@@ -543,7 +546,7 @@ gst_jpegenc_chain (GstPad * pad, GstBuffer * buf)
   GstJpegEnc *jpegenc;
   guchar *data;
   gulong size;
-  guint height, width;
+  guint height;
   guchar *base[3], *end[3];
   gint i, j, k;
 
@@ -568,7 +571,6 @@ gst_jpegenc_chain (GstPad * pad, GstBuffer * buf)
   gst_buffer_copy_metadata (jpegenc->output_buffer, buf,
       GST_BUFFER_COPY_TIMESTAMPS);
 
-  width = jpegenc->width;
   height = jpegenc->height;
 
   for (i = 0; i < jpegenc->channels; i++) {
