@@ -41,8 +41,8 @@
  * for RTP implementations where the contents of the UDP packets is transfered
  * out-of-bounds using SDP or other means.
  *
- * The #GstUDPSrc:buffer property is used to change the default kernel buffer
- * sizes used for receiving packets. The buffer size may be increased for
+ * The #GstUDPSrc:buffer-size property is used to change the default kernel
+ * buffersizes used for receiving packets. The buffer size may be increased for
  * high-volume connections, or may be decreased to limit the possible backlog of
  * incoming data. The system places an absolute limit on these values, on Linux,
  * for example, the default buffer size is typically 50K and can be increased to
@@ -50,7 +50,7 @@
  *
  * The #GstUDPSrc:skip-first-bytes property is used to strip off an arbitrary
  * number of bytes from the start of the raw udp packet and can be used to strip
- * off proprietary header, for example. 
+ * off proprietary header, for example.
  *
  * The udpsrc is always a live source. It does however not provide a #GstClock,
  * this is left for upstream elements such as an RTP session manager or demuxer
@@ -78,7 +78,7 @@
  * because it is blocked by a firewall.
  * </para>
  * <para>
- * A custom file descriptor can be configured with the 
+ * A custom file descriptor can be configured with the
  * #GstUDPSrc:sockfd property. The socket will be closed when setting the
  * element to READY by default. This behaviour can be
  * overriden with the #GstUDPSrc:closefd property, in which case the application
@@ -117,9 +117,6 @@
 #endif
 
 #include <gst/netbuffer/gstnetbuffer.h>
-#ifdef G_OS_WIN32
-typedef int socklen_t;
-#endif
 
 #ifdef HAVE_FIONREAD_IN_SYS_FILIO
 #include <sys/filio.h>
@@ -145,6 +142,7 @@ static GstStaticPadTemplate src_template = GST_STATIC_PAD_TEMPLATE ("src",
 #define UDP_DEFAULT_CLOSEFD            TRUE
 #define UDP_DEFAULT_SOCK                -1
 #define UDP_DEFAULT_AUTO_MULTICAST     TRUE
+#define UDP_DEFAULT_REUSE              TRUE
 
 enum
 {
@@ -162,6 +160,7 @@ enum
   PROP_CLOSEFD,
   PROP_SOCK,
   PROP_AUTO_MULTICAST,
+  PROP_REUSE,
 
   PROP_LAST
 };
@@ -219,8 +218,7 @@ gst_udpsrc_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&src_template));
+  gst_element_class_add_static_pad_template (element_class, &src_template);
 
   gst_element_class_set_details_simple (element_class, "UDP packet receiver",
       "Source/Network",
@@ -247,50 +245,60 @@ gst_udpsrc_class_init (GstUDPSrcClass * klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PORT,
       g_param_spec_int ("port", "Port",
           "The port to receive the packets from, 0=allocate", 0, G_MAXUINT16,
-          UDP_DEFAULT_PORT, G_PARAM_READWRITE));
+          UDP_DEFAULT_PORT, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_MULTICAST_GROUP,
       g_param_spec_string ("multicast-group", "Multicast Group",
           "The Address of multicast group to join", UDP_DEFAULT_MULTICAST_GROUP,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_MULTICAST_IFACE,
       g_param_spec_string ("multicast-iface", "Multicast Interface",
           "The network interface on which to join the multicast group",
-          UDP_DEFAULT_MULTICAST_IFACE, G_PARAM_READWRITE));
+          UDP_DEFAULT_MULTICAST_IFACE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_URI,
       g_param_spec_string ("uri", "URI",
           "URI in the form of udp://multicast_group:port", UDP_DEFAULT_URI,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_CAPS,
       g_param_spec_boxed ("caps", "Caps",
-          "The caps of the source pad", GST_TYPE_CAPS, G_PARAM_READWRITE));
+          "The caps of the source pad", GST_TYPE_CAPS,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_SOCKFD,
       g_param_spec_int ("sockfd", "Socket Handle",
           "Socket to use for UDP reception. (-1 == allocate)",
-          -1, G_MAXINT, UDP_DEFAULT_SOCKFD, G_PARAM_READWRITE));
+          -1, G_MAXINT, UDP_DEFAULT_SOCKFD,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_BUFFER_SIZE,
       g_param_spec_int ("buffer-size", "Buffer Size",
           "Size of the kernel receive buffer in bytes, 0=default", 0, G_MAXINT,
-          UDP_DEFAULT_BUFFER_SIZE, G_PARAM_READWRITE));
+          UDP_DEFAULT_BUFFER_SIZE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_TIMEOUT,
       g_param_spec_uint64 ("timeout", "Timeout",
           "Post a message after timeout microseconds (0 = disabled)", 0,
-          G_MAXUINT64, UDP_DEFAULT_TIMEOUT, G_PARAM_READWRITE));
+          G_MAXUINT64, UDP_DEFAULT_TIMEOUT,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (G_OBJECT_CLASS (klass),
       PROP_SKIP_FIRST_BYTES, g_param_spec_int ("skip-first-bytes",
           "Skip first bytes", "number of bytes to skip for each udp packet", 0,
-          G_MAXINT, UDP_DEFAULT_SKIP_FIRST_BYTES, G_PARAM_READWRITE));
+          G_MAXINT, UDP_DEFAULT_SKIP_FIRST_BYTES,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_CLOSEFD,
       g_param_spec_boolean ("closefd", "Close sockfd",
           "Close sockfd if passed as property on state change",
-          UDP_DEFAULT_CLOSEFD, G_PARAM_READWRITE));
+          UDP_DEFAULT_CLOSEFD, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_SOCK,
       g_param_spec_int ("sock", "Socket Handle",
           "Socket currently in use for UDP reception. (-1 = no socket)",
-          -1, G_MAXINT, UDP_DEFAULT_SOCK, G_PARAM_READABLE));
+          -1, G_MAXINT, UDP_DEFAULT_SOCK,
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gobject_class, PROP_AUTO_MULTICAST,
       g_param_spec_boolean ("auto-multicast", "Auto Multicast",
           "Automatically join/leave multicast groups",
-          UDP_DEFAULT_AUTO_MULTICAST, G_PARAM_READWRITE));
+          UDP_DEFAULT_AUTO_MULTICAST,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  g_object_class_install_property (gobject_class, PROP_REUSE,
+      g_param_spec_boolean ("reuse", "Reuse", "Enable reuse of the port",
+          UDP_DEFAULT_REUSE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstbasesrc_class->start = gst_udpsrc_start;
   gstbasesrc_class->stop = gst_udpsrc_stop;
@@ -318,6 +326,7 @@ gst_udpsrc_init (GstUDPSrc * udpsrc, GstUDPSrcClass * g_class)
   udpsrc->externalfd = (udpsrc->sockfd != -1);
   udpsrc->auto_multicast = UDP_DEFAULT_AUTO_MULTICAST;
   udpsrc->sock.fd = UDP_DEFAULT_SOCK;
+  udpsrc->reuse = UDP_DEFAULT_REUSE;
 
   /* configure basesrc to be a live source */
   gst_base_src_set_live (GST_BASE_SRC (udpsrc), TRUE);
@@ -523,7 +532,7 @@ no_select:
 
   /* patch pktdata and len when stripping off the headers */
   if (G_UNLIKELY (udpsrc->skip_first_bytes != 0)) {
-    if (G_UNLIKELY (readsize <= udpsrc->skip_first_bytes))
+    if (G_UNLIKELY (readsize < udpsrc->skip_first_bytes))
       goto skip_error;
 
     pktdata += udpsrc->skip_first_bytes;
@@ -693,6 +702,9 @@ gst_udpsrc_set_property (GObject * object, guint prop_id, const GValue * value,
     case PROP_AUTO_MULTICAST:
       udpsrc->auto_multicast = g_value_get_boolean (value);
       break;
+    case PROP_REUSE:
+      udpsrc->reuse = g_value_get_boolean (value);
+      break;
     default:
       break;
   }
@@ -741,6 +753,9 @@ gst_udpsrc_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_AUTO_MULTICAST:
       g_value_set_boolean (value, udpsrc->auto_multicast);
       break;
+    case PROP_REUSE:
+      g_value_set_boolean (value, udpsrc->reuse);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -759,11 +774,7 @@ gst_udpsrc_start (GstBaseSrc * bsrc)
   gint ret;
   int rcvsize;
   struct sockaddr_storage bind_address;
-#ifdef G_OS_WIN32
-  gint len;
-#else
-  guint len;
-#endif
+  socklen_t len;
   src = GST_UDPSRC (bsrc);
 
   if (src->sockfd == -1) {
@@ -782,7 +793,8 @@ gst_udpsrc_start (GstBaseSrc * bsrc)
 
     GST_DEBUG_OBJECT (src, "got socket %d", src->sock.fd);
 
-    reuse = 1;
+    GST_DEBUG_OBJECT (src, "setting reuse %d", src->reuse);
+    reuse = src->reuse ? 1 : 0;
     if ((ret =
             setsockopt (src->sock.fd, SOL_SOCKET, SO_REUSEADDR, &reuse,
                 sizeof (reuse))) < 0)
