@@ -102,8 +102,8 @@ gst_wavparse_base_init (gpointer g_class)
   GstPadTemplate *src_template;
 
   /* register pads */
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&sink_template_factory));
+  gst_element_class_add_static_pad_template (element_class,
+      &sink_template_factory);
 
   src_template = gst_pad_template_new ("wavparse_src", GST_PAD_SRC,
       GST_PAD_SOMETIMES, gst_riff_create_audio_template_caps ());
@@ -1156,6 +1156,8 @@ gst_waveparse_ignore_chunk (GstWavParse * wav, GstBuffer * buf, guint32 tag,
   return TRUE;
 }
 
+#define MAX_BUFFER_SIZE 4096
+
 static GstFlowReturn
 gst_wavparse_stream_headers (GstWavParse * wav)
 {
@@ -1198,7 +1200,8 @@ gst_wavparse_stream_headers (GstWavParse * wav)
 
     if (tag == GST_RIFF_TAG_JUNK || tag == GST_RIFF_TAG_JUNQ ||
         tag == GST_RIFF_TAG_bext || tag == GST_RIFF_TAG_BEXT ||
-        tag == GST_RIFF_TAG_LIST) {
+        tag == GST_RIFF_TAG_LIST || tag == GST_RIFF_TAG_ID32 ||
+        tag == GST_RIFF_TAG_IDVX) {
       GST_DEBUG_OBJECT (wav, "skipping %" GST_FOURCC_FORMAT " chunk",
           GST_FOURCC_ARGS (tag));
       gst_buffer_unref (buf);
@@ -1578,6 +1581,19 @@ gst_wavparse_stream_headers (GstWavParse * wav)
 
   wav->state = GST_WAVPARSE_DATA;
 
+  /* determine reasonable max buffer size,
+   * that is, buffers not too small either size or time wise
+   * so we do not end up with too many of them */
+  /* var abuse */
+  upstream_size = 0;
+  gst_wavparse_time_to_bytepos (wav, 40 * GST_MSECOND, &upstream_size);
+  wav->max_buf_size = upstream_size;
+  wav->max_buf_size = MAX (wav->max_buf_size, MAX_BUFFER_SIZE);
+  if (wav->blockalign > 0)
+    wav->max_buf_size -= (wav->max_buf_size % wav->blockalign);
+
+  GST_DEBUG_OBJECT (wav, "max buffer size %d", wav->max_buf_size);
+
   return GST_FLOW_OK;
 
   /* ERROR */
@@ -1723,6 +1739,27 @@ gst_wavparse_send_event (GstElement * element, GstEvent * event)
   return res;
 }
 
+static gboolean
+gst_wavparse_have_dts_caps (const GstCaps * caps, GstTypeFindProbability prob)
+{
+  GstStructure *s;
+
+  s = gst_caps_get_structure (caps, 0);
+  if (!gst_structure_has_name (s, "audio/x-dts"))
+    return FALSE;
+  if (prob >= GST_TYPE_FIND_LIKELY)
+    return TRUE;
+  /* DTS at non-0 offsets and without second sync may yield POSSIBLE .. */
+  if (prob < GST_TYPE_FIND_POSSIBLE)
+    return FALSE;
+  /* .. in which case we want at least a valid-looking rate and channels */
+  if (!gst_structure_has_field (s, "channels"))
+    return FALSE;
+  /* and for extra assurance we could also check the rate from the DTS frame
+   * against the one in the wav header, but for now let's not do that */
+  return gst_structure_has_field (s, "rate");
+}
+
 static void
 gst_wavparse_add_src_pad (GstWavParse * wav, GstBuffer * buf)
 {
@@ -1738,9 +1775,8 @@ gst_wavparse_add_src_pad (GstWavParse * wav, GstBuffer * buf)
 
       tf_caps = gst_type_find_helper_for_buffer (GST_OBJECT (wav), buf, &prob);
       if (tf_caps != NULL) {
-        s = gst_caps_get_structure (tf_caps, 0);
-        if (gst_structure_has_name (s, "audio/x-dts")
-            && prob >= GST_TYPE_FIND_LIKELY) {
+        GST_LOG ("typefind caps = %" GST_PTR_FORMAT ", P=%d", tf_caps, prob);
+        if (gst_wavparse_have_dts_caps (tf_caps, prob)) {
           GST_INFO_OBJECT (wav, "Found DTS marker in file marked as raw PCM");
           gst_caps_unref (wav->caps);
           wav->caps = tf_caps;
@@ -1782,8 +1818,6 @@ gst_wavparse_add_src_pad (GstWavParse * wav, GstBuffer * buf)
   }
 }
 
-#define MAX_BUFFER_SIZE 4096
-
 static GstFlowReturn
 gst_wavparse_stream_data (GstWavParse * wav)
 {
@@ -1806,7 +1840,7 @@ iterate_adapter:
    * amounts of data regardless of the playback rate */
   desired =
       MIN (gst_guint64_to_gdouble (wav->dataleft),
-      MAX_BUFFER_SIZE * wav->segment.abs_rate);
+      wav->max_buf_size * wav->segment.abs_rate);
 
   if (desired >= wav->blockalign && wav->blockalign > 0)
     desired -= (desired % wav->blockalign);
@@ -2038,36 +2072,39 @@ pause:
     wav->segment_running = FALSE;
     gst_pad_pause_task (pad);
 
-    if (GST_FLOW_IS_FATAL (ret) || ret == GST_FLOW_NOT_LINKED) {
-      if (ret == GST_FLOW_UNEXPECTED) {
-        /* add pad before we perform EOS */
-        if (G_UNLIKELY (wav->first)) {
-          wav->first = FALSE;
-          gst_wavparse_add_src_pad (wav, NULL);
-        }
-        /* perform EOS logic */
-        if (wav->segment.flags & GST_SEEK_FLAG_SEGMENT) {
-          GstClockTime stop;
+    if (ret == GST_FLOW_UNEXPECTED) {
+      /* add pad before we perform EOS */
+      if (G_UNLIKELY (wav->first)) {
+        wav->first = FALSE;
+        gst_wavparse_add_src_pad (wav, NULL);
+      }
 
-          if ((stop = wav->segment.stop) == -1)
-            stop = wav->segment.duration;
+      if (wav->state == GST_WAVPARSE_START)
+        GST_ELEMENT_ERROR (wav, STREAM, WRONG_TYPE,
+            ("No valid input found before end of stream"), (NULL));
 
-          gst_element_post_message (GST_ELEMENT_CAST (wav),
-              gst_message_new_segment_done (GST_OBJECT_CAST (wav),
-                  wav->segment.format, stop));
-        } else {
-          if (wav->srcpad != NULL)
-            gst_pad_push_event (wav->srcpad, gst_event_new_eos ());
-        }
+      /* perform EOS logic */
+      if (wav->segment.flags & GST_SEEK_FLAG_SEGMENT) {
+        GstClockTime stop;
+
+        if ((stop = wav->segment.stop) == -1)
+          stop = wav->segment.duration;
+
+        gst_element_post_message (GST_ELEMENT_CAST (wav),
+            gst_message_new_segment_done (GST_OBJECT_CAST (wav),
+                wav->segment.format, stop));
       } else {
-        /* for fatal errors we post an error message, post the error
-         * first so the app knows about the error first. */
-        GST_ELEMENT_ERROR (wav, STREAM, FAILED,
-            (_("Internal data flow error.")),
-            ("streaming task paused, reason %s (%d)", reason, ret));
         if (wav->srcpad != NULL)
           gst_pad_push_event (wav->srcpad, gst_event_new_eos ());
       }
+    } else if (ret == GST_FLOW_NOT_LINKED || ret < GST_FLOW_UNEXPECTED) {
+      /* for fatal errors we post an error message, post the error
+       * first so the app knows about the error first. */
+      GST_ELEMENT_ERROR (wav, STREAM, FAILED,
+          (_("Internal data flow error.")),
+          ("streaming task paused, reason %s (%d)", reason, ret));
+      if (wav->srcpad != NULL)
+        gst_pad_push_event (wav->srcpad, gst_event_new_eos ());
     }
     return;
   }
@@ -2244,6 +2281,11 @@ gst_wavparse_sink_event (GstPad * pad, GstEvent * event)
         /* stream leftover data in current segment */
         gst_wavparse_flush_data (wav);
       }
+
+      if (wav->state == GST_WAVPARSE_START)
+        GST_ELEMENT_ERROR (wav, STREAM, WRONG_TYPE,
+            ("No valid input found before end of stream"), (NULL));
+
       /* fall-through */
     case GST_EVENT_FLUSH_STOP:
       gst_adapter_clear (wav->adapter);
