@@ -115,21 +115,18 @@ gst_oss4_source_base_init (gpointer g_class)
   templ = gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
       gst_oss4_audio_get_template_caps ());
   gst_element_class_add_pad_template (element_class, templ);
+  gst_object_unref (templ);
 }
 
 static void
 gst_oss4_source_class_init (GstOss4SourceClass * klass)
 {
   GObjectClass *gobject_class;
-  GstElementClass *gstelement_class;
   GstBaseSrcClass *gstbasesrc_class;
-  GstBaseAudioSrcClass *gstbaseaudiosrc_class;
   GstAudioSrcClass *gstaudiosrc_class;
 
   gobject_class = (GObjectClass *) klass;
-  gstelement_class = (GstElementClass *) klass;
   gstbasesrc_class = (GstBaseSrcClass *) klass;
-  gstbaseaudiosrc_class = (GstBaseAudioSrcClass *) klass;
   gstaudiosrc_class = (GstAudioSrcClass *) klass;
 
   gobject_class->dispose = gst_oss4_source_dispose;
@@ -151,12 +148,12 @@ gst_oss4_source_class_init (GstOss4SourceClass * klass)
       g_param_spec_string ("device", "Device",
           "OSS4 device (e.g. /dev/oss/hdaudio0/pcm0 or /dev/dspN) "
           "(NULL = use first available device)",
-          DEFAULT_DEVICE, G_PARAM_READWRITE));
+          DEFAULT_DEVICE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   g_object_class_install_property (gobject_class, PROP_DEVICE_NAME,
       g_param_spec_string ("device-name", "Device name",
           "Human-readable name of the sound device", DEFAULT_DEVICE_NAME,
-          G_PARAM_READABLE));
+          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -928,16 +925,12 @@ static void
 gst_oss4_source_mixer_set_mute (GstMixer * mixer, GstMixerTrack * track,
     gboolean mute)
 {
-  GstOss4Source *oss;
-
   g_return_if_fail (mixer != NULL);
   g_return_if_fail (track != NULL);
   g_return_if_fail (GST_IS_MIXER_TRACK (track));
   g_return_if_fail (GST_IS_OSS4_SOURCE (mixer));
   g_return_if_fail (GST_OSS4_SOURCE_IS_OPEN (mixer));
 
-  oss = GST_OSS4_SOURCE (mixer);
-
   /* FIXME: implement gst_oss4_source_mixer_set_mute() - what to do here? */
   /* oss4_mixer_set_mute (mixer->mixer, track, mute); */
 }
