@@ -44,6 +44,8 @@
 #include <config.h>
 #endif
 
+#undef HAVE_XVIDEO
+
 #include <string.h>
 #include <sys/time.h>
 #include "v4l2src_calls.h"
@@ -51,7 +53,7 @@
 
 #include "gstv4l2colorbalance.h"
 #include "gstv4l2tuner.h"
-#if 0                           /* overlay is still not implemented #ifdef HAVE_XVIDEO */
+#ifdef HAVE_XVIDEO
 #include "gstv4l2xoverlay.h"
 #endif
 #include "gstv4l2vidorient.h"
@@ -63,6 +65,7 @@ GST_DEBUG_CATEGORY (v4l2src_debug);
 
 #define PROP_DEF_QUEUE_SIZE         2
 #define PROP_DEF_ALWAYS_COPY        TRUE
+#define PROP_DEF_DECIMATE           1
 
 #define DEFAULT_PROP_DEVICE   "/dev/video0"
 
@@ -71,13 +74,14 @@ enum
   PROP_0,
   V4L2_STD_OBJECT_PROPS,
   PROP_QUEUE_SIZE,
-  PROP_ALWAYS_COPY
+  PROP_ALWAYS_COPY,
+  PROP_DECIMATE
 };
 
 GST_IMPLEMENT_V4L2_PROBE_METHODS (GstV4l2SrcClass, gst_v4l2src);
 GST_IMPLEMENT_V4L2_COLOR_BALANCE_METHODS (GstV4l2Src, gst_v4l2src);
 GST_IMPLEMENT_V4L2_TUNER_METHODS (GstV4l2Src, gst_v4l2src);
-#if 0                           /* overlay is still not implemented #ifdef HAVE_XVIDEO */
+#ifdef HAVE_XVIDEO
 GST_IMPLEMENT_V4L2_XOVERLAY_METHODS (GstV4l2Src, gst_v4l2src);
 #endif
 GST_IMPLEMENT_V4L2_VIDORIENT_METHODS (GstV4l2Src, gst_v4l2src);
@@ -90,7 +94,7 @@ gst_v4l2src_iface_supported (GstImplementsInterface * iface, GType iface_type)
 {
   GstV4l2Object *v4l2object = GST_V4L2SRC (iface)->v4l2object;
 
-#if 0                           /* overlay is still not implemented #ifdef HAVE_XVIDEO */
+#ifdef HAVE_XVIDEO
   g_assert (iface_type == GST_TYPE_TUNER ||
       iface_type == GST_TYPE_X_OVERLAY ||
       iface_type == GST_TYPE_COLOR_BALANCE ||
@@ -104,7 +108,7 @@ gst_v4l2src_iface_supported (GstImplementsInterface * iface, GType iface_type)
   if (v4l2object->video_fd == -1)
     return FALSE;
 
-#if 0                           /* overlay is still not implemented #ifdef HAVE_XVIDEO */
+#ifdef HAVE_XVIDEO
   if (iface_type == GST_TYPE_X_OVERLAY && !GST_V4L2_IS_OVERLAY (v4l2object))
     return FALSE;
 #endif
@@ -140,7 +144,8 @@ gst_v4l2src_init_interfaces (GType type)
     NULL,
     NULL,
   };
-#if 0                           /* overlay is still not implemented #ifdef HAVE_XVIDEO */
+#ifdef HAVE_XVIDEO
+  /* FIXME: does GstXOverlay for v4l2src make sense in a GStreamer context? */
   static const GInterfaceInfo v4l2_xoverlay_info = {
     (GInterfaceInitFunc) gst_v4l2src_xoverlay_interface_init,
     NULL,
@@ -167,7 +172,7 @@ gst_v4l2src_init_interfaces (GType type)
   g_type_add_interface_static (type,
       GST_TYPE_IMPLEMENTS_INTERFACE, &v4l2iface_info);
   g_type_add_interface_static (type, GST_TYPE_TUNER, &v4l2_tuner_info);
-#if 0                           /* overlay is still not implemented #ifdef HAVE_XVIDEO */
+#ifdef HAVE_XVIDEO
   g_type_add_interface_static (type, GST_TYPE_X_OVERLAY, &v4l2_xoverlay_info);
 #endif
   g_type_add_interface_static (type,
@@ -216,6 +221,7 @@ gst_v4l2src_base_init (gpointer g_class)
 {
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
   GstV4l2SrcClass *gstv4l2src_class = GST_V4L2SRC_CLASS (g_class);
+  GstPadTemplate *pad_template;
 
   gstv4l2src_class->v4l2_class_devices = NULL;
 
@@ -227,10 +233,11 @@ gst_v4l2src_base_init (gpointer g_class)
       "Edgard Lima <edgard.lima@indt.org.br>,"
       " Stefan Kost <ensonic@users.sf.net>");
 
-  gst_element_class_add_pad_template
-      (gstelement_class,
+  pad_template =
       gst_pad_template_new ("src", GST_PAD_SRC, GST_PAD_ALWAYS,
-          gst_v4l2_object_get_all_caps ()));
+      gst_v4l2_object_get_all_caps ());
+  gst_element_class_add_pad_template (gstelement_class, pad_template);
+  gst_object_unref (pad_template);
 }
 
 static void
@@ -264,6 +271,17 @@ gst_v4l2src_class_init (GstV4l2SrcClass * klass)
       g_param_spec_boolean ("always-copy", "Always Copy",
           "If the buffer will or not be used directly from mmap",
           PROP_DEF_ALWAYS_COPY, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+  /**
+   * GstV4l2Src:decimate
+   *
+   * Only use every nth frame
+   *
+   * Since: 0.10.26
+   */
+  g_object_class_install_property (gobject_class, PROP_DECIMATE,
+      g_param_spec_int ("decimate", "Decimate",
+          "Only use every nth frame", 1, G_MAXINT,
+          PROP_DEF_DECIMATE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   basesrc_class->get_caps = GST_DEBUG_FUNCPTR (gst_v4l2src_get_caps);
   basesrc_class->set_caps = GST_DEBUG_FUNCPTR (gst_v4l2src_set_caps);
@@ -290,6 +308,7 @@ gst_v4l2src_init (GstV4l2Src * v4l2src, GstV4l2SrcClass * klass)
   v4l2src->num_buffers = PROP_DEF_QUEUE_SIZE;
 
   v4l2src->always_copy = PROP_DEF_ALWAYS_COPY;
+  v4l2src->decimate = PROP_DEF_DECIMATE;
 
   v4l2src->is_capturing = FALSE;
 
@@ -338,6 +357,9 @@ gst_v4l2src_set_property (GObject * object,
       case PROP_ALWAYS_COPY:
         v4l2src->always_copy = g_value_get_boolean (value);
         break;
+      case PROP_DECIMATE:
+        v4l2src->decimate = g_value_get_int (value);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -361,6 +383,9 @@ gst_v4l2src_get_property (GObject * object,
       case PROP_ALWAYS_COPY:
         g_value_set_boolean (value, v4l2src->always_copy);
         break;
+      case PROP_DECIMATE:
+        g_value_set_int (value, v4l2src->decimate);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -555,7 +580,7 @@ gst_v4l2src_get_caps (GstBaseSrc * src)
 
   ret = gst_caps_new_empty ();
 
-  for (walk = v4l2src->v4l2object->formats; walk; walk = walk->next) {
+  for (walk = formats; walk; walk = walk->next) {
     struct v4l2_fmtdesc *format;
 
     GstStructure *template;
@@ -591,6 +616,7 @@ gst_v4l2src_set_caps (GstBaseSrc * src, GstCaps * caps)
 {
   GstV4l2Src *v4l2src;
   gint w = 0, h = 0;
+  gboolean interlaced;
   struct v4l2_fmtdesc *format;
   guint fps_n, fps_d;
   guint size;
@@ -612,8 +638,8 @@ gst_v4l2src_set_caps (GstBaseSrc * src, GstCaps * caps)
 
   /* we want our own v4l2 type of fourcc codes */
   if (!gst_v4l2_object_get_caps_info (v4l2src->v4l2object, caps, &format, &w,
-          &h, &fps_n, &fps_d, &size)) {
-    GST_DEBUG_OBJECT (v4l2src,
+          &h, &interlaced, &fps_n, &fps_d, &size)) {
+    GST_INFO_OBJECT (v4l2src,
         "can't get capture format from caps %" GST_PTR_FORMAT, caps);
     return FALSE;
   }
@@ -621,8 +647,8 @@ gst_v4l2src_set_caps (GstBaseSrc * src, GstCaps * caps)
   GST_DEBUG_OBJECT (v4l2src, "trying to set_capture %dx%d at %d/%d fps, "
       "format %s", w, h, fps_n, fps_d, format->description);
 
-  if (!gst_v4l2src_set_capture (v4l2src, format->pixelformat, w, h, fps_n,
-          fps_d))
+  if (!gst_v4l2src_set_capture (v4l2src, format->pixelformat, w, h,
+          interlaced, fps_n, fps_d))
     /* error already posted */
     return FALSE;
 
@@ -710,6 +736,10 @@ gst_v4l2src_start (GstBaseSrc * src)
 
   v4l2src->offset = 0;
 
+  /* activate settings for first frame */
+  v4l2src->ctrl_time = 0;
+  gst_object_sync_values (G_OBJECT (src), v4l2src->ctrl_time);
+
   return TRUE;
 }
 
@@ -800,6 +830,10 @@ gst_v4l2src_get_read (GstV4l2Src * v4l2src, GstBuffer ** buf)
   gint buffersize;
 
   buffersize = v4l2src->frame_byte_size;
+  /* In case the size per frame is unknown assume it's a streaming format (e.g.
+   * mpegts) and grab a reasonable default size instead */
+  if (buffersize == 0)
+    buffersize = GST_BASE_SRC (v4l2src)->blocksize;
 
   *buf = gst_buffer_new_and_alloc (buffersize);
 
@@ -906,9 +940,19 @@ static GstFlowReturn
 gst_v4l2src_create (GstPushSrc * src, GstBuffer ** buf)
 {
   GstV4l2Src *v4l2src = GST_V4L2SRC (src);
+  int i;
   GstFlowReturn ret;
 
+  for (i = 0; i < v4l2src->decimate - 1; i++) {
+    ret = v4l2src->get_frame (v4l2src, buf);
+    if (ret != GST_FLOW_OK) {
+      return ret;
+    }
+    gst_buffer_unref (*buf);
+  }
+
   ret = v4l2src->get_frame (v4l2src, buf);
+
   /* set buffer metadata */
   if (G_LIKELY (ret == GST_FLOW_OK && *buf)) {
     GstClock *clock;
@@ -918,6 +962,7 @@ gst_v4l2src_create (GstPushSrc * src, GstBuffer ** buf)
     GST_BUFFER_OFFSET_END (*buf) = v4l2src->offset;
 
     /* timestamps, LOCK to get clock and base time. */
+    /* FIXME: element clock and base_time is rarely changing */
     GST_OBJECT_LOCK (v4l2src);
     if ((clock = GST_ELEMENT_CLOCK (v4l2src))) {
       /* we have a clock, get base time and ref clock */
@@ -943,6 +988,19 @@ gst_v4l2src_create (GstPushSrc * src, GstBuffer ** buf)
       }
     }
 
+    /* activate settings for next frame */
+    if (GST_CLOCK_TIME_IS_VALID (v4l2src->duration)) {
+      v4l2src->ctrl_time += v4l2src->duration;
+    } else {
+      /* this is not very good (as it should be the next timestamp),
+       * still good enough for linear fades (as long as it is not -1) 
+       */
+      v4l2src->ctrl_time = timestamp;
+    }
+    gst_object_sync_values (G_OBJECT (src), v4l2src->ctrl_time);
+    GST_INFO_OBJECT (src, "sync to %" GST_TIME_FORMAT,
+        GST_TIME_ARGS (v4l2src->ctrl_time));
+
     /* FIXME: use the timestamp from the buffer itself! */
     GST_BUFFER_TIMESTAMP (*buf) = timestamp;
     GST_BUFFER_DURATION (*buf) = v4l2src->duration;
