@@ -69,7 +69,10 @@ enum
   PROP_STARTX,
   PROP_STARTY,
   PROP_ENDX,
-  PROP_ENDY
+  PROP_ENDY,
+  PROP_REMOTE,
+  PROP_XID,
+  PROP_XNAME,
 };
 
 GST_BOILERPLATE (GstXImageSrc, gst_ximage_src, GstPushSrc, GST_TYPE_PUSH_SRC);
@@ -98,11 +101,41 @@ gst_ximage_src_return_buf (GstXImageSrc * ximagesrc,
     /* need to increment the refcount again to recycle */
     gst_buffer_ref (GST_BUFFER (ximage));
     g_mutex_lock (ximagesrc->pool_lock);
+    GST_BUFFER_FLAGS (GST_BUFFER (ximage)) = 0; /* clear out any flags from the previous use */
     ximagesrc->buffer_pool = g_slist_prepend (ximagesrc->buffer_pool, ximage);
     g_mutex_unlock (ximagesrc->pool_lock);
   }
 }
 
+static Window
+gst_ximage_src_find_window (GstXImageSrc * src, Window root, const char *name)
+{
+  Window *children;
+  Window window = 0, root_return, parent_return;
+  unsigned int nchildren;
+  char *tmpname;
+  int n, status;
+
+  status = XFetchName (src->xcontext->disp, root, &tmpname);
+  if (status && !strcmp (name, tmpname))
+    return root;
+
+  status =
+      XQueryTree (src->xcontext->disp, root, &root_return, &parent_return,
+      &children, &nchildren);
+  if (!status || !children)
+    return (Window) 0;
+
+  for (n = 0; n < nchildren; ++n) {
+    window = gst_ximage_src_find_window (src, children[n], name);
+    if (window != 0)
+      break;
+  }
+
+  XFree (children);
+  return window;
+}
+
 static gboolean
 gst_ximage_src_open_display (GstXImageSrc * s, const gchar * name)
 {
@@ -123,8 +156,51 @@ gst_ximage_src_open_display (GstXImageSrc * s, const gchar * name)
   s->width = s->xcontext->width;
   s->height = s->xcontext->height;
 
-  /* Always capture root window, for now */
   s->xwindow = s->xcontext->root;
+  if (s->xid != 0 || s->xname) {
+    int status;
+    XWindowAttributes attrs;
+    Window window;
+
+    if (s->xid != 0) {
+      status = XGetWindowAttributes (s->xcontext->disp, s->xid, &attrs);
+      if (status) {
+        GST_DEBUG_OBJECT (s, "Found window XID %" G_GUINT64_FORMAT, s->xid);
+        s->xwindow = s->xid;
+        goto window_found;
+      } else {
+        GST_WARNING_OBJECT (s, "Failed to get window %" G_GUINT64_FORMAT
+            " attributes", s->xid);
+      }
+    }
+
+    if (s->xname) {
+      GST_DEBUG_OBJECT (s, "Looking for window %s", s->xname);
+      window = gst_ximage_src_find_window (s, s->xcontext->root, s->xname);
+      if (window != 0) {
+        GST_DEBUG_OBJECT (s, "Found window named %s, ", s->xname);
+        status = XGetWindowAttributes (s->xcontext->disp, window, &attrs);
+        if (status) {
+          s->xwindow = window;
+          goto window_found;
+        } else {
+          GST_WARNING_OBJECT (s, "Failed to get window attributes for "
+              "window named %s", s->xname);
+        }
+      }
+    }
+
+    GST_INFO_OBJECT (s, "Using root window");
+    goto use_root_window;
+
+  window_found:
+    g_assert (s->xwindow != 0);
+    s->width = attrs.width;
+    s->height = attrs.height;
+    GST_INFO_OBJECT (s, "Using default window size of %dx%d",
+        s->width, s->height);
+  }
+use_root_window:
 
 #ifdef HAVE_XFIXES
   /* check if xfixes supported */
@@ -465,10 +541,10 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
             if (ximagesrc->endx > ximagesrc->startx &&
                 ximagesrc->endy > ximagesrc->starty) {
               /* see if damage area intersects */
-              if (rects[i].x + rects[i].width < ximagesrc->startx ||
+              if (rects[i].x + rects[i].width - 1 < ximagesrc->startx ||
                   rects[i].x > ximagesrc->endx) {
                 /* trivial reject */
-              } else if (rects[i].y + rects[i].height < ximagesrc->starty ||
+              } else if (rects[i].y + rects[i].height - 1 < ximagesrc->starty ||
                   rects[i].y > ximagesrc->endy) {
                 /* trivial reject */
               } else {
@@ -479,12 +555,12 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
                     rects[i].x;
                 starty = (rects[i].y < ximagesrc->starty) ? ximagesrc->starty :
                     rects[i].y;
-                width = (rects[i].x + rects[i].width < ximagesrc->endx) ?
+                width = (rects[i].x + rects[i].width - 1 < ximagesrc->endx) ?
                     rects[i].x + rects[i].width - startx :
-                    ximagesrc->endx - startx;
-                height = (rects[i].y + rects[i].height < ximagesrc->endy) ?
+                    ximagesrc->endx - startx + 1;
+                height = (rects[i].y + rects[i].height - 1 < ximagesrc->endy) ?
                     rects[i].y + rects[i].height - starty : ximagesrc->endy -
-                    starty;
+                    starty + 1;
 
                 GST_LOG_OBJECT (ximagesrc,
                     "Retrieving damaged sub-region @ %d,%d size %dx%d as intersect region",
@@ -591,9 +667,16 @@ gst_ximage_src_ximage_get (GstXImageSrc * ximagesrc)
 #endif /* HAVE_XSHM */
     {
       GST_DEBUG_OBJECT (ximagesrc, "Retrieving screen using XGetImage");
-      ximage->ximage = XGetImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
-          ximagesrc->startx, ximagesrc->starty, ximagesrc->width,
-          ximagesrc->height, AllPlanes, ZPixmap);
+      if (ximagesrc->remote) {
+        XGetSubImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
+            ximagesrc->startx, ximagesrc->starty, ximagesrc->width,
+            ximagesrc->height, AllPlanes, ZPixmap, ximage->ximage, 0, 0);
+      } else {
+        ximage->ximage =
+            XGetImage (ximagesrc->xcontext->disp, ximagesrc->xwindow,
+            ximagesrc->startx, ximagesrc->starty, ximagesrc->width,
+            ximagesrc->height, AllPlanes, ZPixmap);
+      }
     }
 #ifdef HAVE_XDAMAGE
   }
@@ -824,6 +907,24 @@ gst_ximage_src_set_property (GObject * object, guint prop_id,
     case PROP_ENDY:
       src->endy = g_value_get_uint (value);
       break;
+    case PROP_REMOTE:
+      src->remote = g_value_get_boolean (value);
+      break;
+    case PROP_XID:
+      if (src->xcontext != NULL) {
+        g_warning ("ximagesrc window ID must be set before opening display");
+        break;
+      }
+      src->xid = g_value_get_uint64 (value);
+      break;
+    case PROP_XNAME:
+      if (src->xcontext != NULL) {
+        g_warning ("ximagesrc window name must be set before opening display");
+        break;
+      }
+      g_free (src->xname);
+      src->xname = g_strdup (g_value_get_string (value));
+      break;
     default:
       break;
   }
@@ -864,6 +965,15 @@ gst_ximage_src_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_ENDY:
       g_value_set_uint (value, src->endy);
       break;
+    case PROP_REMOTE:
+      g_value_set_boolean (value, src->remote);
+      break;
+    case PROP_XID:
+      g_value_set_uint64 (value, src->xid);
+      break;
+    case PROP_XNAME:
+      g_value_set_string (value, src->xname);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -896,7 +1006,7 @@ gst_ximage_src_base_init (gpointer g_class)
       "Lutz Mueller <lutz@users.sourceforge.net>, "
       "Jan Schmidt <thaytan@mad.scientist.com>, "
       "Zaheer Merali <zaheerabbas at merali dot org>");
-  gst_element_class_add_pad_template (ec, gst_static_pad_template_get (&t));
+  gst_element_class_add_static_pad_template (ec, &t);
 }
 
 static void
@@ -916,6 +1026,7 @@ gst_ximage_src_finalize (GObject * object)
   if (src->xcontext)
     ximageutil_xcontext_clear (src->xcontext);
 
+  g_free (src->xname);
   g_mutex_free (src->pool_lock);
   g_mutex_free (src->x_lock);
 
@@ -940,30 +1051,46 @@ gst_ximage_src_get_caps (GstBaseSrc * bs)
             (s)->srcpad));
 
   xcontext = s->xcontext;
+  width = s->xcontext->width;
+  height = s->xcontext->height;
+  if (s->xwindow != 0) {
+    XWindowAttributes attrs;
+    int status = XGetWindowAttributes (s->xcontext->disp, s->xwindow, &attrs);
+    if (status) {
+      width = attrs.width;
+      height = attrs.height;
+    }
+  }
 
-  width = xcontext->width;
-  height = xcontext->height;
-  if (s->endx > s->startx && s->endy > s->starty) {
+  /* property comments say 0 means right/bottom, means we can't capture
+     the top left pixel alone */
+  if (s->endx == 0)
+    s->endx = width - 1;
+  if (s->endy == 0)
+    s->endy = height - 1;
+
+  if (s->endx >= s->startx && s->endy >= s->starty) {
     /* this means user has put in values */
     if (s->startx < xcontext->width && s->endx < xcontext->width &&
-        s->starty < xcontext->height && s->endy < xcontext->height) {
+        s->starty < xcontext->height && s->endy < xcontext->height &&
+        s->startx >= 0 && s->starty >= 0) {
       /* values are fine */
-      s->width = width = s->endx - s->startx;
-      s->height = height = s->endy - s->starty;
+      s->width = width = s->endx - s->startx + 1;
+      s->height = height = s->endy - s->starty + 1;
     } else {
       GST_WARNING
           ("User put in co-ordinates overshooting the X resolution, setting to full screen");
       s->startx = 0;
       s->starty = 0;
-      s->endx = 0;
-      s->endy = 0;
+      s->endx = width - 1;
+      s->endy = height - 1;
     }
   } else {
     GST_WARNING ("User put in bogus co-ordinates, setting to full screen");
     s->startx = 0;
     s->starty = 0;
-    s->endx = 0;
-    s->endy = 0;
+    s->endx = width - 1;
+    s->endy = height - 1;
   }
   GST_DEBUG ("width = %d, height=%d", width, height);
   return gst_caps_new_simple ("video/x-raw-rgb",
@@ -1033,14 +1160,14 @@ gst_ximage_src_class_init (GstXImageSrcClass * klass)
 
   g_object_class_install_property (gc, PROP_DISPLAY_NAME,
       g_param_spec_string ("display-name", "Display", "X Display Name", NULL,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gc, PROP_SCREEN_NUM,
       g_param_spec_uint ("screen-num", "Screen number", "X Screen Number",
-          0, G_MAXINT, 0, G_PARAM_READWRITE));
+          0, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   g_object_class_install_property (gc, PROP_SHOW_POINTER,
       g_param_spec_boolean ("show-pointer", "Show Mouse Pointer",
           "Show mouse pointer (if XFixes extension enabled)", TRUE,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
    * GstXImageSrc:use-damage
    *
@@ -1051,7 +1178,7 @@ gst_ximage_src_class_init (GstXImageSrcClass * klass)
   g_object_class_install_property (gc, PROP_USE_DAMAGE,
       g_param_spec_boolean ("use-damage", "Use XDamage",
           "Use XDamage (if XDamage extension enabled)", TRUE,
-          G_PARAM_READWRITE));
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
    * GstXImageSrc:startx
    *
@@ -1063,7 +1190,7 @@ gst_ximage_src_class_init (GstXImageSrcClass * klass)
   g_object_class_install_property (gc, PROP_STARTX,
       g_param_spec_uint ("startx", "Start X co-ordinate",
           "X coordinate of top left corner of area to be recorded (0 for top left of screen)",
-          0, G_MAXINT, 0, G_PARAM_READWRITE));
+          0, G_MAXINT, 0, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
   /**
    * GstXImageSrc:starty
    *
@@ -1101,6 +1228,43 @@ gst_ximage_src_class_init (GstXImageSrcClass * klass)
           "Y coordinate of bottom right corner of area to be recorded (0 for bottom right of screen)",
           0, G_MAXINT, 0, G_PARAM_READWRITE));
 
+  /**
+   * GstXImageSrc:remote
+   *
+   * Whether the X display is remote. The element will try to use alternate calls
+   * known to work better with remote displays.
+   *
+   * Since: 0.10.26
+   **/
+  g_object_class_install_property (gc, PROP_REMOTE,
+      g_param_spec_boolean ("remote", "Remote dispay",
+          "Whether the display is remote", FALSE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstXImageSrc:xid
+   *
+   * The XID of the window to capture. 0 for the root window (default).
+   *
+   * Since: 0.10.31
+   **/
+  g_object_class_install_property (gc, PROP_XID,
+      g_param_spec_uint64 ("xid", "Window XID",
+          "Window XID to capture from", 0, G_MAXUINT64, 0,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
+  /**
+   * GstXImageSrc:xname
+   *
+   * The name of the window to capture, if any.
+   *
+   * Since: 0.10.31
+   **/
+  g_object_class_install_property (gc, PROP_XNAME,
+      g_param_spec_string ("xname", "Window name",
+          "Window name to capture from", NULL,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+
   parent_class = g_type_class_peek_parent (klass);
 
   push_class->create = gst_ximage_src_create;
@@ -1127,6 +1291,7 @@ gst_ximage_src_init (GstXImageSrc * ximagesrc, GstXImageSrcClass * klass)
   ximagesrc->starty = 0;
   ximagesrc->endx = 0;
   ximagesrc->endy = 0;
+  ximagesrc->remote = FALSE;
 }
 
 static gboolean
